from openai import OpenAI
from config import SYSTEM_PROMPT
from text_processing import fuzzy_correct_chinese

client = OpenAI()

def transcribe(audio_data):
    try:
        transcript = client.audio.transcriptions.create(
            file=audio_data,
            model="whisper-1",
            response_format="text"
        )
        return transcript
    except Exception as e:
        print(f"轉錄時發生錯誤：{str(e)}")
        return None

def post_process_transcript(transcript, temperature=0):
    corrected_transcript = fuzzy_correct_chinese(transcript)
    
    messages = [
        {"role": "system", "content": SYSTEM_PROMPT},
        {"role": "user", "content": f"請校對並修正以下轉錄文本，但不要改變其原意或回答問題：\n\n{corrected_transcript}"}
    ]

    response = client.chat.completions.create(
        model="gpt-4",
        temperature=temperature,
        messages=messages
    )

    return response.choices[0].message.content

def process_audio(audio_data):
    raw_transcript = transcribe(audio_data)
    if raw_transcript is None:
        return None, None
    corrected_transcript = post_process_transcript(raw_transcript)
    return raw_transcript, corrected_transcript